// Name
// Student ID

// Define the sizes of the hole and ball.
int hole_radius = 15;
int ball_radius = 10;

// In many games, the action occurs in distinct phases.  In
// this mini-putt game, you're either sitting idle in between
// strokes, or you're winding up for a stroke, or the ball is
// in motion, or you're celebrating the completion of a level
// before returning to the idle state.  Define some constants
// to help us distinguish which state we're in.  We'll never
// change these values -- they're "symbolic names" that we can
// use in the rest of the program.
int STATE_READY = 1;
int STATE_WINDUP = 2;
int STATE_SIMULATE = 3;
int STATE_CELEBRATE = 4;

// Now use a global variable to remember what the state is at
// this very moment.
int state;

// When the user drags out a line to initiate a golf shot, that
// line will have a particular length.  We want to cap the smallest
// and largest that length can be, so that the user doesn't 
// cause the ball to shot off at light speed.
float min_len = 50;
float max_len = 300;
// Then we'll map the stroke length to an actual speed.
float min_speed = 3;
float max_speed = 15;

// The location of the hole
float hole_x;
float hole_y;

// The current location of the ball.
float ball_x;
float ball_y;
// And the current velocity.
float vx;
float vy;

// Friction, modelled as the amount of speed lost every frame.
float friction = 0.1;

void setup()
{
  size(600, 500);
  noStroke();
  strokeWeight(2);
 
  resetGame();
}

void resetGame()
{
  ball_x = random( ball_radius, width - ball_radius );
  ball_y = random( ball_radius, height - ball_radius );
  
  // Try to find a hole that is at least 200 pixels away from the 
  // ball.
  for( int idx = 0; idx < 20; ++idx ) {
    hole_x = random( hole_radius, width - hole_radius );
    hole_y = random( hole_radius, height - hole_radius );
    if( dist( ball_x, ball_y, hole_x, hole_y ) > 200.0 ) {
      // Yay, found a far away hole!  Stop looping.
      break;
    }
  }
    
  state = STATE_READY;
}

void drawBoard()
{
  background(95, 200, 70);
  fill(0);
  ellipse(hole_x, hole_y, hole_radius*2, hole_radius*2);
  fill(255);
  ellipse(ball_x, ball_y, ball_radius*2, ball_radius*2);
}

// Do the actual physical simulation.  As usual, this 
// depends on a few global values: the current position
// and velocity of the ball, and the friction with the
// putting green.  Don't forget to process collisions --
// that means you'll also be referring to the width
// and height of the sketch, and the size of the ball
// and hole.
void step()
{
  // 1. Calculate the ball's speed -- the length of the
  // velocity vector.  Use the built-in mag() function.
  
  // 2. If the speed is lower than the amount that would
  // be lost due to friction (i.e., the speed is less
  // than the friction), then simply stop the ball.  
  // Set the ball's velocity to (0,0), set the state
  // to STATE_READY, and leave the function.
  
  // 3. Otherwise, we can modify the current velocity to
  // take friction into account.  We need to modify 
  // the vector (vx,vy) so that its length gets reduced
  // precisely by the amount of friction.  The easiest
  // way to do that is multiply both vx and vy by 
  // (1-friction/speed).
  
  // 4. Once you've updated velocities, you're ready to
  // use them to update positions (i.e., implement the
  // First Law).
  
  // 5. Next, check for collisions with the four edges of
  // the sketch.  If any collisions occur, adjust the 
  // velocity vector by flipping one of its coordinates.
  
  // 6. Check if the ball is inside the hole.  If so, 
  // it indicates successful completion of this level.
  if( dist( ball_x, ball_y, hole_x, hole_y ) < hole_radius ) {
    state = STATE_CELEBRATE;
  }
}

void draw()
{
  if( state == STATE_READY ) {
    drawBoard();
  } else if( state == STATE_CELEBRATE ) {
    // The way this state is written, we immediately reset and
    // start a new level.  But we left "CELEBRATE" mode here
    // in case you would like to implement some kind of 
    // animation to indicate that the current level was
    // solved.
    resetGame();
  } else if( state == STATE_WINDUP ) {
    drawBoard();
    stroke(200);
    line(ball_x, ball_y, mouseX, mouseY);
    noStroke();
  } else if( state == STATE_SIMULATE ) {
    step();
    drawBoard();
  }
}

void mousePressed()
{
  // There's only one mouse press we care about -- we're at
  // the start of a stroke, and the user clicks on the ball
  // to begin a stroke.
  
  if( state == STATE_READY ) {
    // Hit test against ball.
    if( dist( mouseX, mouseY, ball_x, ball_y ) < ball_radius ) {
      state = STATE_WINDUP;
    }
  }
}

void mouseReleased()
{
  // If you're in the middle of winding up a stroke,
  // Turn it into an actual stroke (putt) by imparting
  // some initial velocity to the ball.
  
  if( state == STATE_WINDUP ) {
    float len = dist( mouseX, mouseY, ball_x, ball_y );
    len = constrain( len, min_len, max_len );
    float speed = map( len, min_len, max_len, min_speed, max_speed );
    
    vx = (ball_x-mouseX) / len * speed;
    vy = (ball_y-mouseY) / len * speed;
    
    state = STATE_SIMULATE;
  }
}